// Copyright (c) 1984-2006 Sangoma Technologies Inc.
//
// Sample program for reading/writing configuration
// of Sangoma Card Driver. Program reads current configuration,
// if it set to T1, then configration is changed to E1 (and vice versa)
// after that, the driver is restarted.
//
// WARNING: extra care should be taken when modifing system registry.
//
// Target: Windows XP/2003 and later. Win32 Console.
//
//
#include <windows.h>
#include <setupapi.h>	//for SetupDiXXX() functions
#include <commctrl.h>
#include <shellapi.h>
#include <devguid.h>	//for GUID_DEVCLASS_NET

#define __WINDOWS__		//must be defined before including
						//cross-platform header files.

#include <stdio.h>					//for sprintf()
#include <stdlib.h>					//for atoi()
#include <conio.h>

#include <wanpipe_defines.h>		//for 'wan_udp_hdr_t'
#include <sang_status_defines.h>	//return codes
#include <sang_api.h>				//for IOCTL codes
#include <sdla_te1.h>				//for TE1 macros
#include <sdla_56k.h>				//for 56k macros
#include <sdla_bri.h>				//for BRI macros
#include <sdla_remora.h>			//Analog card
#include <sdla_te3.h>				//T3 card
#include <sdla_front_end.h>			//for front-end (T1/E1/56k) commands
#include <wanpipe_cfg.h>			//for WAN_MEDIA_NONE...

/* definitions */
#define MAX_COMP_INSTID 2096
#define MAX_COMP_DESC   2096
#define MAX_FRIENDLY    2096	
#define TMP_BUFFER_LEN	500

#define T1_BIT_RATE		1536000
#define E1_BIT_RATE		1984000

#define DBG_PRNT	if(1)printf

const char *SearchedDeviceName = "  Sangoma Hardware Abstraction Driver";

/* globals */
TCHAR	name[MAX_FRIENDLY];

char    szCompInstanceId[MAX_COMP_INSTID];
TCHAR   szCompDescription[MAX_COMP_DESC];
DWORD   dwRegType;

unsigned int TargetBitrate = 0;

/* function prototypes */
void GetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue, DWORD * pdwSize);
void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue);
void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, int iValue);
void get_set_cfg_and_disable();
void enable();

/////////////////////////////////////////////////////////////////////////////////

void GetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue, DWORD * pdwSize)
{	
	//reading twice to set dwTemp to needed value
	RegQueryValueEx(hkey, szKeyname, NULL, NULL, (unsigned char *)szValue, pdwSize);
	RegQueryValueEx(hkey, szKeyname, NULL, NULL, (unsigned char *)szValue, pdwSize);

	DBG_PRNT("GetRegistryStringValue(): %s: %s\n", szKeyname, szValue);
}

void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue)
{
	DWORD	dwSize;
	
	dwSize = strlen(szValue)+1;
    RegSetValueEx(hkey, szKeyname, 0, REG_SZ, (unsigned char *)szValue, dwSize);

	DBG_PRNT("SetRegistryStringValue(): %s: %s\n", szKeyname, szValue);
}

void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, int iValue)
{
	DWORD	dwSize;
	char	szTemp[TMP_BUFFER_LEN];

	sprintf(szTemp, "%u", iValue);
	dwSize = strlen(szTemp)+1;
    RegSetValueEx(hkey, szKeyname, 0, REG_SZ, (unsigned char *)szTemp, dwSize);

	DBG_PRNT("SetRegistryStringValue(): %s: %s\n", szKeyname, szTemp);
}

/////////////////////////////////////////////////////////////////////////////////

void print_info()
{
	int i;
	int need_reload=0; // if registry setting doesn't match to what we need
	int num=1;
	SP_DEVINFO_DATA deid={sizeof(SP_DEVINFO_DATA)};
	HDEVINFO hdi = SetupDiGetClassDevs((struct _GUID *)&GUID_DEVCLASS_NET, NULL,NULL, DIGCF_PRESENT); 
	char WanpipeName[100];//DeviceName
	DWORD dwTemp;
	HKEY hkey;
	char szTmp[MAX_FRIENDLY];

	//Possible Port Names:
	//Sangoma Hardware Abstraction Driver (Port 1)
	//Sangoma Hardware Abstraction Driver (Port 2)
	//Sangoma Hardware Abstraction Driver (Port 3)
	//Sangoma Hardware Abstraction Driver (Port 4)
	//Sangoma Hardware Abstraction Driver (Port 5)
	//Sangoma Hardware Abstraction Driver (Port 6)
	//Sangoma Hardware Abstraction Driver (Port 7)
	//Sangoma Hardware Abstraction Driver (Port 8)

	//this version will search for a known port:
	//sprintf(name,"  Sangoma Hardware Abstraction Driver (Port %d)",num);

	//this version will search for all ports:
	sprintf(name, SearchedDeviceName);

	for (i=0;SetupDiEnumDeviceInfo(hdi,i,&deid);i++) 
	{

		BOOL fSuccess = SetupDiGetDeviceInstanceId(hdi, &deid, (PSTR)szCompInstanceId,MAX_COMP_INSTID, NULL);
		if (!fSuccess){
			continue;
		}

		fSuccess =SetupDiGetDeviceRegistryProperty(hdi, &deid,SPDRP_DEVICEDESC,&dwRegType, (BYTE*)szCompDescription,MAX_COMP_DESC,NULL);
		if (!fSuccess){
			continue;
		}

		// check if it is right Sangoma board and channel #
		if (strncmp(szCompDescription,name,strlen(name)) !=0) {	// Windows can add #2 etc - do NOT consider
			//not a Sangoma port
			continue;
		}

		printf("\n* %s\n", szCompDescription);


		if(TRUE == SetupDiGetDeviceRegistryProperty(hdi, &deid, SPDRP_LOCATION_INFORMATION, &dwRegType, (BYTE*)szTmp, MAX_FRIENDLY, NULL)){
			printf("SPDRP_LOCATION_INFORMATION: %s\n", szTmp);
		}

		if(TRUE == SetupDiGetDeviceRegistryProperty(hdi, &deid, SPDRP_UI_NUMBER, &dwRegType, (BYTE*)&dwTemp, sizeof(dwTemp), NULL)){
			printf("SPDRP_UI_NUMBER: %i\n", dwTemp);
		}
		
		hkey = SetupDiOpenDevRegKey(hdi, &deid, DICS_FLAG_GLOBAL, 0, DIREG_DRV, KEY_READ | KEY_SET_VALUE );
		if(hkey == INVALID_HANDLE_VALUE){
			printf("Error: Failed to open Registry key!!\n");
			continue;
		}

		WanpipeName[0] = '\n';
		GetRegistryStringValue(hkey, "DeviceName", WanpipeName, &dwTemp);
		printf("Wanpipe Name: %s\n", WanpipeName);


	}//for()
	SetupDiDestroyDeviceInfoList(hdi);
}

int main()
{
	print_info();
	return 0;
}
